{% if user.is_anonymous %}
  <p>Hello, guest | <a href="{% url auth_login %}">Log in</a></p>
{% else %}
  <p>Hello, {{user.username}} | <a href="{% url webidprovider-logout %}">Log out</a></p>
{% endif %}
