{% extends "django_webid/provider/base_site.html" %}

{% load i18n %}

{% block extra_head_base %}
    <link rel="stylesheet" href="{{ STATIC_URL }}django_webid/provider/css/site_tabs.css" />
    {% block extra_head %}{% endblock %}
{% endblock %}

{% block login %}
{% include "django_webid/provider/login_header.html" %}
{% endblock %}

{% block extra_body_base %}
    {% block extra_body %}{% endblock %}
{% endblock %}
