{% extends "django_webid/provider/base.html" %}
{% load i18n %}
{% load uni_form_tags %}

{% block head_title %}{% blocktrans %}WebID{% endblocktrans %}{% endblock %}

{% block extra_head %}
{% endblock %}
{% block tabhead %}
{% endblock %}


{% block body %}
<div id="home-header">
	<!-- HEADER title -->
	<div class="header-title">
	  <div id="home-logo"></div>
	  {# XXX get Site Name #}
	  <h1>Django WebID Provider</h1>
	  <p class="tagline">Your Identity. Your data. Your rules.</p>
        </div>
        <!-- END HEADER title -->
        <!-- HEADER buttons -->
        <div class="home-header-buttons">
	  {% if user.is_anonymous %}
	  <a class="reg" href="{% url webidprovider-create_user%}">Create WebID</a>
	  <div class="log-box">
		  <p>or <a class="login" href="{% url auth_login %}">Log in</a></p>
	  </div>
	  {% else %}
	          <a class="reg" href="{% url webidprovider-add_cert %}">Create WebID</a>
	  <div class="log-box">
		  <p>Hello {{user.username}} | <a href="{% url webidprovider-logout %}">Logout</a></p>
  	  </div>
	  {% endif %}
	  </div>
	</div>
</div>
<div id="home-content">
	<div class="left">
		{% if user.is_anonymous %}
		<h2>What is this?</h2>
		<p>This site is a demo of <b>django-webid-provider</b>, a reusable django app that turns any django-powered site into a <a href='http://www.w3.org/wiki/WebID'>WebID</a> provider. Don't know what is WebID good for?</p>
		<p>You'll also probably want to use the companion authentication package <b>django-webid-auth</b>, and <b>django-webid-profile</b> for semantic-ready user profiles (<em>coming soon! ...</em>)
		{% else %}
		<h2>Your Actions</h2>
		  <p><a href="{% url webidprovider-add_cert %}">Create New WebID Cert</a></p>
		  <p><a href="{% url webidprovider-cert_list %}">Manage your WebID Certs</a></p>
		  <p><a href="#">Edit your Profile</a></p>
		{% endif %} {# anonymous #}
		<h3>For developers</h3>
		<p>Fork code <a href="http://github.com/bennomadic/django-webid-provider">on github</a>. Check the <a href="#">docs</a>. Have a look at the current <a href='http://www.w3.org/2005/Incubator/webid/spec/'>WebID spec.</a></p>
	</div>
	<div class="right">
		<div id="semantic-machine"></div>
	</div>
</div>
{% endblock %}

{% block extra_body %}
{% endblock %}

