
{% load i18n %}

<p>
  {% blocktrans %}
    <b>WebID certificate generator</b> is an 
    <a href="http://djangoproject.com/">Django</a> app intended create
    <a href="http://www.w3.org/2005/Incubator/webid/spec/"> WebID</a>
    profiles for authentication.
  {% endblocktrans %}
</p>
